# SPDX-FileCopyrightText: 2024 Luca Weiss
# SPDX-License-Identifier: GPL-3.0-only

from gi.repository import Gtk
from gi.repository import Adw
from . import esim
from . import pages
from . import widgets
import os


@Gtk.Template(resource_path="/eu/lucaweiss/lpa-gtk/mainwindow.ui")
class MainWindow(Adw.ApplicationWindow):
    __gtype_name__ = "LpaGtkMainWindow"

    stack = Gtk.Template.Child()
    sim_slot_button = Gtk.Template.Child()
    profile_list_box = Gtk.Template.Child()
    profile_add_button = Gtk.Template.Child()
    nav_view = Gtk.Template.Child()
    nav_home = Gtk.Template.Child()
    nav_profile = Gtk.Template.Child()
    nav_download = Gtk.Template.Child()
    detail_page = Gtk.Template.Child()
    download_page = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        if os.getenv("LPA_GTK_BACKEND") == "dummy":
            backends = [esim.DummyBackend(1), esim.DummyBackend(2)]
        else:
            backends = []
            for slot in [1, 2]:
                backend = esim.LpacBackend(slot)
                if backend.is_operational():
                    backends.append(backend)

        if not backends:
            self.stack.set_visible_child_name("no-backend")
            self.profile_add_button.hide()
            self.sim_slot_button.hide()
            return

        sim_slot_model = Gtk.StringList()
        for backend in backends:
            slot = backend.get_slot()
            print(f"Available slot: {slot}")
            sim_slot_model.append(str(slot))
        self.sim_slot_button.set_model(sim_slot_model)
        # Work around tickmark not appearing on first item in dropdown.
        # Not sure if this is a bug in Gtk/Libadwaita or intended behavior.
        self.sim_slot_button.set_selected(Gtk.INVALID_LIST_POSITION)
        self.sim_slot_button.set_selected(0)

        self.active_slot = backends[0].get_slot()
        self.download_page.set_active_slot(self.active_slot)
        self.detail_page.set_active_slot(self.active_slot)
        print(f"Initial active slot: {self.active_slot}")

        self.esim_manager = esim.ESimManager(backends)

        self.profile_add_button.connect("clicked", lambda *_: self.nav_view.push(self.nav_download))

        self.sim_slot_button.connect("notify::selected-item", self.sim_slot_changed)

        self.profile_list_box.connect("row-activated", self.profile_selected)

        self.nav_home.connect("showing", lambda *_: self.update())
        self.nav_download.connect("hidden", lambda *_: self.download_page.reset())

        self.detail_page.connect("profile-deleted", lambda *_: self.nav_view.pop())

        self.download_page.set_manager(self.esim_manager)
        self.detail_page.set_manager(self.esim_manager)

        self.update()

    def sim_slot_changed(self, combo_row, _prop):
        self.active_slot = int(combo_row.get_selected_item().get_string())
        self.download_page.set_active_slot(self.active_slot)
        self.detail_page.set_active_slot(self.active_slot)
        print(f"Selected SIM slot: {self.active_slot}")
        self.update()

    def profile_selected(self, _list_box, profile_widget):
        profile = profile_widget.profile

        self.detail_page.set_profile(profile)
        self.nav_view.push(self.nav_profile)

    def update(self):
        print("Updating eSIM profile list...")
        self.profile_list_box.remove_all()

        profiles = self.esim_manager.get_profiles(self.active_slot)
        for profile in profiles:
            w = widgets.ESimProfileWidget(
                title=profile.nickname or profile.name,
                enabled=profile.state == esim.ESimProfileState.ENABLED,
                provider=profile.service_provider_name,
                iccid=profile.iccid,
            )
            w.profile = profile
            self.profile_list_box.append(w)

        if len(profiles):
            self.stack.set_visible_child_name("list")
        else:
            self.stack.set_visible_child_name("empty")
